@@ -19,22 +19,16 @@ package EmailReminder::YearlyEvent;
 
 use strict;
 use warnings;
-
+use overload '""' => \&str;
 use Date::Manip;
 use POSIX;
 
-use EmailReminder::Event;
 use EmailReminder::Utils;
 
-require Exporter;
-our @ISA    = ("EmailReminder::Event", "Exporter");
-our @EXPORT = qw(get_th get_name get_original_date);
+use base qw(EmailReminder::Event);
 
-# XML tags, attributes and values
-my $DAY_TAG   = 'day';
-my $MONTH_TAG = 'month';
-my $YEAR_TAG  = 'year';
-my $NAME_TAG  = 'name';
+# XML tags
+__PACKAGE__->mk_accessors(qw(day month year));
 
 # Global date variables
 my $current_time = ParseDate("now");
@@ -42,6 +36,11 @@ my $current_date = ParseDate(UnixDate($current_time, "\%x"));
 my $current_year = UnixDate($current_time, "\%Y");
 my $leap_year    = "1980";
 
+sub str {
+    my ($self) = @_;
+    return $self->get_type . ':' . $self->get_id . ') ' . $self->get_name . ' - ' . $self->get_date;
+}
+
 # Hard-coded value for this event's type (class method)
 sub get_type
 {
@@ -51,39 +50,54 @@ sub get_type
 # Number of fields this event adds to its parent (class method)
 sub get_nb_fields
 {
-    return EmailReminder::Event->get_nb_fields() + 2;
+    my ($self) = @_;
+    return $self->SUPER::get_nb_fields() + 2;
 }
 
-sub get_name
+sub valid_day
 {
-    my ($this) = @_;
-    my $name  = EmailReminder::Utils::get_node_value($this->{XML_NODE}, $NAME_TAG);
-    return $name;
+    my ($class, $new_value) = @_;
+
+    return 1 unless defined $new_value;
+    $new_value = int($new_value);
+    return $new_value if ($new_value >= 1 and $new_value <= 31);
+    return 1;
 }
 
-sub set_name
+sub valid_month
 {
-    my ($this, $new_value) = @_;
-    return EmailReminder::Utils::set_node_value($this->{XML_NODE}, $NAME_TAG, $new_value);
+    my ($class, $new_value) = @_;
+
+    return 1 unless defined $new_value;
+    $new_value = int($new_value);
+    return $new_value if ($new_value >= 1 and $new_value <= 12);
+    return 1;
+}
+
+sub valid_year
+{
+    my ($class, $new_value) = @_;
+    return 0 unless defined $new_value;
+    return int($new_value);
 }
 
 sub get_date
 {
-    my ($this) = @_;
+    my ($self) = @_;
 
-    my $day   = EmailReminder::Utils::get_node_value($this->{XML_NODE}, $DAY_TAG);
-    my $month = EmailReminder::Utils::get_node_value($this->{XML_NODE}, $MONTH_TAG);
-    my $year  = EmailReminder::Utils::get_node_value($this->{XML_NODE}, $YEAR_TAG);
+    my $day   = $self->get_day;
+    my $month = $self->get_month;
+    my $year  = $self->get_year;
 
     # Hack to support events that don't have a starting year
     my $actual_date;
     if (defined($year)) {
-        $this->{"MISSING_YEAR"} = 0;
+        $self->{"MISSING_YEAR"} = 0;
         $actual_date = UnixDate(ParseDate($year ."-".$month."-".$day),
                                 "\%Y-\%m-\%d");
     }
     elsif (defined($month) && defined($day)) {
-        $this->{"MISSING_YEAR"} = 1;
+        $self->{"MISSING_YEAR"} = 1;
         my $full_date = UnixDate(ParseDate($leap_year."-".$month."-".$day), 
                                  "\%Y-\%m-\%d");
         $actual_date = UnixDate(ParseDate($full_date), "\%m-\%d");
@@ -94,7 +108,7 @@ sub get_date
 
 sub set_date
 {
-    my ($this, $new_value) = @_;
+    my ($self, $new_value) = @_;
     my $date;
 
     # Normalize the date entered by the user
@@ -126,14 +140,10 @@ sub set_date
         my @parts = split /-/, $date;
         my $day = pop(@parts);
         my $month = pop(@parts);
-        my $year = pop(@parts) if @parts;
+        my $year = undef;
+        $year = pop(@parts) if @parts;
 
-        return (EmailReminder::Utils::set_node_value($this->{XML_NODE}, 
-                                                     $DAY_TAG, $day) and 
-                EmailReminder::Utils::set_node_value($this->{XML_NODE}, 
-                                                     $MONTH_TAG, $month) and
-                EmailReminder::Utils::set_node_value($this->{XML_NODE}, 
-                                                     $YEAR_TAG, $year));
+        return ($self->set_day($day) and $self->set_month($month) and $self->set_year($year));
     } else {
         return 0;
     }
@@ -141,37 +151,32 @@ sub set_date
 
 sub get_original_date
 {
-    my ($this) = @_;
-    my $date_string = $this->get_date();
-    return undef unless $date_string;
+    my ($self) = @_;
+    my $date_string = $self->get_date();
+    return unless $date_string;
 
-    $date_string = $leap_year."-".$date_string if $this->{"MISSING_YEAR"};
+    $date_string = $leap_year."-".$date_string if $self->{"MISSING_YEAR"};
     return ParseDate($date_string);
 }
 
 sub get_subject
 {
-    my $this = shift;
-    return $this->get_name();
+    my $self = shift;
+    return $self->get_name();
 }
 
 sub get_message_body
 {
-    my $this = shift;
-
-    # destination user
-    my $first_name = shift;
+    my $self = shift;
 
     # event details
-    my $when      = $this->{"WHEN"};
-    my $name      = $this->get_name();
-    my $occurence = $this->get_occurence();
-    my $th        = get_th($occurence);
+    my $when      = $self->{"WHEN"};
+    my $name      = $self->get_name();
+    my $occurence = $self->get_occurence();
+    my $th        = EmailReminder::Utils::get_th($occurence);
     my $event     = defined($occurence) ? "${occurence}$th $name" : $name;
     
-    my $message = <<MESSAGEEND;
-Hi $first_name,
-
+    my $message = <<"MESSAGEEND";
 I just want to remind you of the following event $when:
 
 $event
@@ -183,12 +188,12 @@ MESSAGEEND
 # Returns the occurence number of this event (starts at 1)
 sub get_occurence
 {
-    my $this = shift;
+    my $self = shift;
 
     my $exact_age;
-    unless ($this->{"MISSING_YEAR"}) {
-        my $original_date = $this->get_original_date();
-        return undef unless $original_date;
+    unless ($self->{"MISSING_YEAR"}) {
+        my $original_date = $self->get_original_date();
+        return unless $original_date;
 
         my $delta = DateCalc($original_date, $current_date, 1);
         $exact_age = Delta_Format($delta, 5, '%yd') + 1;
@@ -197,33 +202,14 @@ sub get_occurence
     return defined($exact_age) ? ceil($exact_age) : undef;
 }
 
-# Returns the proper English qualifier for this number
-sub get_th
-{
-    my $number = shift;
-    return undef unless defined($number);
-
-    if ($number >= 11 && $number <= 13) {
-        return "th";
-    } elsif ($number % 10 == 1) {
-        return "st";
-    } elsif ($number % 10 == 2) {
-        return "nd";
-    } elsif ($number % 10 == 3) {
-        return "rd";
-    } else {
-        return "th";
-    }
-}
-
 # Returns 1 if the event will occur in X days (X is a param)
 sub will_occur
 {
-    my $this = shift;
+    my $self = shift;
     my $modifier = shift;
     
     # Apply the modifier to the event date
-    my $modified_date = $this->get_original_date();
+    my $modified_date = $self->get_original_date();
     return 0 unless $modified_date;
 
     if ($modifier) {
